==> gnome-subtitles-0.1.tar.gz: gnome-subtitles-0.1/sublib-0.4/src/SubLib/Persistency/FileInputOutput.cs <==
/*
 * This file is part of SubLib, a library for managing movie subtitles.
 * Copyright (C) 2005-2006 Pedro Castro
 *
 * SubLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SubLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

using System;
using System.IO;
using System.Text;
using org.mozilla.intl.chardet;

namespace SubLib {

internal class FileInputOutput {

	/// <exception cref="UnknownEncodingException">Thrown if the encoding couldn't be auto-detected.</exception>
	internal static string ReadFile (string fileName, out Encoding encoding) {
		StreamReader fileReader = OpenFileForReading(fileName, out encoding);
		string text = Read(fileReader);
		return text;
	}
	
	internal static string ReadFile (string fileName, Encoding encoding, bool detectBOM) {
		StreamReader reader = OpenFileForReading(fileName, encoding, detectBOM);
		string text = Read(reader);
		return text;
	}

	internal static string ReadFile (string fileName, out Encoding encoding, Encoding fallbackEncoding, bool detectBOM) {
		try {
			string text = ReadFile(fileName, out encoding);
			VerboseConsole.WriteLine("Encoding " + encoding + " was auto-detected.");
			return text;
		}
		catch (UnknownEncodingException) {
			VerboseConsole.WriteLine("Could not auto-detect the encoding. Will fallback to " + fallbackEncoding + (detectBOM ? " if no BOM is detected." : String.Empty));
			encoding = fallbackEncoding;
			return ReadFile(fileName, fallbackEncoding, detectBOM);
		}
	}

	internal static void WriteFile (string fileName, string text){
		StreamWriter writer = OpenFileForWriting(fileName);
		try {
			writer.Write(text);
		}
		finally {
			writer.Close();
		}
	}
	
	internal static void WriteFile (string fileName, string text, Encoding encoding) {
		StreamWriter writer = OpenFileForWriting(fileName, encoding);
		try {
			writer.Write(text);
		}
		finally {
			writer.Close();
		}
	}

	
	/* Private members */

	/// <summary>Opens a file for reading, detecting its character encoding.</summary>
	/// <exception cref="UnknownEncodingException">Thrown if the encoding couldn't be auto-detected.</exception>
	private static StreamReader OpenFileForReading (string fileName, out Encoding encoding) {
		FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
		encoding = DetectEncoding(fileStream);
		if (encoding == null)
			throw new UnknownEncodingException();
		else
			return new StreamReader(fileStream, encoding);
	}

	/// <summary>Opens a file for reading, given its encoding.</summary>
	private static StreamReader OpenFileForReading (string fileName, Encoding encoding, bool detectBOM) {
		FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
   		StreamReader fileReader = new StreamReader(fileStream, encoding, detectBOM);
		return fileReader;
	}
	
	private static StreamWriter OpenFileForWriting (string fileName) {
		FileStream file = new FileStream(fileName, FileMode.Create, FileAccess.Write, FileShare.None);
		StreamWriter fileWriter = new StreamWriter(file);
		return fileWriter;	
	}
	
	private static StreamWriter OpenFileForWriting (string fileName, Encoding encoding) {
		FileStream file = new FileStream(fileName, FileMode.Create, FileAccess.Write, FileShare.None);
		StreamWriter fileWriter = new StreamWriter(file, encoding);
		return fileWriter;	
	}

	private static string Read (StreamReader reader) {
		string text = String.Empty;
		try {
			text = reader.ReadToEnd();
		}
		finally {
			reader.Close();
		}
		text = ConvertNewLinesToUnix(text);
		return text;
	}
	
	/// <summary>Detects the encoding by reading a specified stream.</summary>
	/// <remarks>The stream reader position is reset after reading.</remarks>
	/// <returns>The detected Encoding, or null if none was detected.</returns>
	private static Encoding DetectEncoding (Stream stream){
		const int BUFFERSIZE = 1024;
		nsDetector detector = new nsDetector(nsPSMDetector.ALL);
		detector.Init(null);

  		byte[] buffer = new byte[BUFFERSIZE] ;
		int readLength = 0;
		bool finished = false;

		while (!finished) {
			readLength = stream.Read(buffer, 0, buffer.Length);
			if (readLength == 0)
				break;

			finished = detector.DoIt(buffer, readLength, false);
		}
		detector.Done();
		stream.Seek(0, SeekOrigin.Begin);

		string[] detectedEncodings = detector.getProbableCharsets();
		VerboseConsole.WriteLine(DetectedEncodingsToString(detectedEncodings));
		
		string encoding = detectedEncodings[0];
		if (encoding == "nomatch")
			return null;
		
		return GetEncodingFromName(encoding);
	}
	
	private static Encoding GetEncodingFromName (string encoding) {
		switch (encoding) {
			case "UTF-8": return Encoding.UTF8;
			case "windows-1252": return Encoding.GetEncoding(1252);
			case "UTF-16LE": return Encoding.Unicode;
			case "UTF-16BE": return Encoding.BigEndianUnicode;
			case "Shift_JIS": return Encoding.GetEncoding(932);
			case "Big5": return Encoding.GetEncoding(950);
			case "ISO-2022-KR": return Encoding.GetEncoding(949);
			case "ISO-2022-JP": return Encoding.GetEncoding(932);
			case "ISO-2022-CN": return Encoding.GetEncoding(950);
			case "HZ-GB-2312": return Encoding.GetEncoding(936);
			case "GB2312": return Encoding.GetEncoding(936);
			case "GB18030": return Encoding.GetEncoding(936);
			case "x-euc-tw": return Encoding.GetEncoding(51932);
			case "EUC-KR": return Encoding.GetEncoding(949);
			case "EUC-JP": return Encoding.GetEncoding(51932);
			default: return null;
		}
	}
	
	private static string DetectedEncodingsToString (string[] detectedEncodings) {
		string result = "Detected encodings:";
		foreach (string encoding in detectedEncodings) {
			result += " " + encoding;
		}
		return result;
	}
	
	/// <summary>Replaces the occurrences of Windows and Mac newline chars with unix newline.</summary>
	private static string ConvertNewLinesToUnix (string text) {
		text = text.Replace("\r\n", "\n"); //Replace Windows newline
		text = text.Replace("\r", "\n"); //Replace Mac newline
		return text;
	}

}

}

==> gnome-subtitles-0.4.tar.gz: gnome-subtitles-0.4/sublib-0.6/src/SubLib/Persistency/FileInputOutput.cs <==
/*
 * This file is part of SubLib, a library for managing movie subtitles.
 * Copyright (C) 2005-2007 Pedro Castro
 *
 * SubLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SubLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

using System;
using System.IO;
using System.Text;
using org.mozilla.intl.chardet;

namespace SubLib {

internal class FileInputOutput {

	/// <exception cref="UnknownEncodingException">Thrown if the encoding couldn't be auto-detected.</exception>
	internal static string ReadFile (string fileName, out Encoding encoding) {
		StreamReader fileReader = OpenFileForReading(fileName, out encoding);
		string text = Read(fileReader);
		return text;
	}
	
	internal static string ReadFile (string fileName, Encoding encoding, bool detectBOM) {
		StreamReader reader = OpenFileForReading(fileName, encoding, detectBOM);
		string text = Read(reader);
		return text;
	}

	internal static string ReadFile (string fileName, out Encoding encoding, Encoding fallbackEncoding, bool detectBOM) {
		try {
			string text = ReadFile(fileName, out encoding);
			VerboseConsole.WriteLine("Encoding " + encoding + " was auto-detected.");
			return text;
		}
		catch (UnknownEncodingException) {
			VerboseConsole.WriteLine("Could not auto-detect the encoding. Will fallback to " + fallbackEncoding + (detectBOM ? " if no BOM is detected." : String.Empty));
			encoding = fallbackEncoding;
			return ReadFile(fileName, fallbackEncoding, detectBOM);
		}
	}

	internal static void WriteFile (string fileName, string text){
		StreamWriter writer = OpenFileForWriting(fileName);
		try {
			writer.Write(text);
		}
		finally {
			writer.Close();
		}
	}
	
	internal static void WriteFile (string fileName, string text, Encoding encoding) {
		StreamWriter writer = OpenFileForWriting(fileName, encoding);
		try {
			writer.Write(text);
		}
		finally {
			writer.Close();
		}
	}

	
	/* Private members */

	/// <summary>Opens a file for reading, detecting its character encoding.</summary>
	/// <exception cref="UnknownEncodingException">Thrown if the encoding couldn't be auto-detected.</exception>
	private static StreamReader OpenFileForReading (string fileName, out Encoding encoding) {
		FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
		encoding = DetectEncoding(fileStream);
		if (encoding == null)
			throw new UnknownEncodingException();
		else
			return new StreamReader(fileStream, encoding);
	}

	/// <summary>Opens a file for reading, given its encoding.</summary>
	private static StreamReader OpenFileForReading (string fileName, Encoding encoding, bool detectBOM) {
		FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
   		StreamReader fileReader = new StreamReader(fileStream, encoding, detectBOM);
		return fileReader;
	}
	
	private static StreamWriter OpenFileForWriting (string fileName) {
		FileStream file = new FileStream(fileName, FileMode.Create, FileAccess.Write, FileShare.None);
		StreamWriter fileWriter = new StreamWriter(file);
		return fileWriter;	
	}
	
	private static StreamWriter OpenFileForWriting (string fileName, Encoding encoding) {
		FileStream file = new FileStream(fileName, FileMode.Create, FileAccess.Write, FileShare.None);
		StreamWriter fileWriter = new StreamWriter(file, encoding);
		return fileWriter;	
	}

	private static string Read (StreamReader reader) {
		string text = String.Empty;
		try {
			text = reader.ReadToEnd();
		}
		finally {
			reader.Close();
		}
		text = ConvertNewLinesToUnix(text);
		return text;
	}
	
	/// <summary>Detects the encoding by reading a specified stream.</summary>
	/// <remarks>The stream reader position is reset after reading.</remarks>
	/// <returns>The detected Encoding, or null if none was detected.</returns>
	private static Encoding DetectEncoding (Stream stream){
		const int BUFFERSIZE = 1024;
		nsDetector detector = new nsDetector(nsPSMDetector.ALL);
		detector.Init(null);

  		byte[] buffer = new byte[BUFFERSIZE] ;
		int readLength = 0;
		bool finished = false;

		while (!finished) {
			readLength = stream.Read(buffer, 0, buffer.Length);
			if (readLength == 0)
				break;

			finished = detector.DoIt(buffer, readLength, false);
		}
		detector.Done();
		stream.Seek(0, SeekOrigin.Begin);

		string[] detectedEncodings = detector.getProbableCharsets();
		VerboseConsole.WriteLine(DetectedEncodingsToString(detectedEncodings));
		
		string encoding = detectedEncodings[0];
		if (encoding == "nomatch")
			return null;
		
		int codePage = GetCodePage(encoding);
		if (codePage != -1)
			return Encoding.GetEncoding(codePage);
		else
			return null;
	}
	
	// Note: ISO-2022-CN, HZ-GB-2312 and x-euc-tw are not defined as their code pages were not found
	private static int GetCodePage (string encoding) {
		switch (encoding) {
			case "UTF-8": return 65001;
			case "windows-1252": return 1252;
			case "UTF-16LE": return 1200;
			case "UTF-16BE": return 1201;
			case "Shift_JIS": return 932;
			case "Big5": return 950;
			case "ISO-2022-KR": return 50225;
			case "ISO-2022-JP": return 50221;
			case "GB2312": return 936;
			case "GB18030": return 54936;
			case "EUC-KR": return 51949;
			case "EUC-JP": return 51932;
			default: return -1;
		}
	}
	
	private static string DetectedEncodingsToString (string[] detectedEncodings) {
		string result = "Detected encodings:";
		foreach (string encoding in detectedEncodings) {
			result += " " + encoding;
		}
		return result;
	}
	
	/// <summary>Replaces the occurrences of Windows and Mac newline chars with unix newline.</summary>
	private static string ConvertNewLinesToUnix (string text) {
		text = text.Replace("\r\n", "\n"); //Replace Windows newline
		text = text.Replace("\r", "\n"); //Replace Mac newline
		return text;
	}

}

}
